@@ -151,8 +151,8 @@ pub use imp::thread::tls::StartupTlsInfo;
 ///
 ///  - Acquiring any other locks that are held in other threads on the parent
 ///    at the time of the `fork`, as the child only contains one thread, and
-///    attempting to acquire such locks will deadlock (though this is
-///    [not considered unsafe]).
+///    attempting to acquire such locks will deadlock (though this is [not
+///    considered unsafe]).
 ///
 ///  - Performing any dynamic allocation using the global allocator, since
 ///    global allocators may use locks to ensure thread safety, and their locks
@@ -220,13 +220,13 @@ pub unsafe fn fork() -> io::Result<Option<Pid>> {
     imp::syscalls::fork()
 }
 
-/// `execveat(path.as_z_str(), args, env_vars)`—Execute a new command using
-/// the current process.
+/// `execveat(dirfd, path.as_z_str(), argv, envp, flags)`—Execute a new
+/// command using the current process.
 ///
 /// # Safety
 ///
-/// The `args` and `env_vars` slices must be NUL-terminated, and their contents
-/// must be pointers to NUL-terminated byte arrays.
+/// The `argv` and `envp` pointers must point to NUL-terminated arrays, and
+/// their contents must be pointers to NUL-terminated byte arrays.
 ///
 /// # References
 ///  - [Linux]
@@ -244,3 +244,21 @@ pub unsafe fn execveat<Fd: AsFd>(
     let dirfd = dirfd.as_fd();
     imp::syscalls::execveat(dirfd, path, argv, envp, flags)
 }
+
+/// `execve(path.as_z_str(), argv, envp)`—Execute a new command using the
+/// current process.
+///
+/// # Safety
+///
+/// The `argv` and `envp` pointers must point to NUL-terminated arrays, and
+/// their contents must be pointers to NUL-terminated byte arrays.
+///
+/// # References
+///  - [Linux]
+///
+/// [Linux]: https://man7.org/linux/man-pages/man2/execve.2.html
+#[inline]
+#[cfg(linux_raw)]
+pub unsafe fn execve(path: &ZStr, argv: *const *const u8, envp: *const *const u8) -> io::Error {
+    imp::syscalls::execve(path, argv, envp)
+}
